import psutil
import tuned.monitors
import tuned.logs

log = tuned.logs.get()


class ProcessMonitor(tuned.monitors.Monitor):
    app_program_dict = {
        "mysql": ["mysqld"],
        "redis": ["redis-server"],
        "nginx": ["nginx"],
        "unixbench": ["Run"],
        "unixbench-arithoh": ["arithoh"],
        "unixbench-context1": ["context1"],
        "unixbench-dhry2": ["dhry2"],
        "unixbench-dhry2reg": ["dhry2reg"],
        "unixbench-double": ["double"],
        "unixbench-execl": ["execl"],
        "unixbench-float": ["float"],
        "unixbench-fstime": ["fstime"],
        "unixbench-gfx-x11": ["gfx-x11"],
        "unixbench-hanoi": ["hanoi"],
        "unixbench-int": ["int"],
        "unixbench-long": ["long"],
        "unixbench-looper": ["looper"],
        "unixbench-pipe": ["pipe"],
        "unixbench-register": ["register"],
        "unixbench-short": ["short"],
        "unixbench-spawn": ["spawn"],
        "unixbench-syscall": ["syscall"],
        "unixbench-whetstone-double": ["whetstone-double"],
        "fio": ["fio"],
        "iozone": ["iozone"],
        "lmbench": ["lmbench"],
        "netperf": ["netperf"]
    }

    pid_set = set()
    pid_app_dict = {}

    @classmethod
    def _init_available_devices(cls):
        cls._available_devices = set(["application"])
        cls._load["application"] = set()

    @classmethod
    def update(cls):
        cur_pids = set(psutil.pids())
        prev_pids = cls.pid_set

        # collect new pid and gone pid
        new_pids = cur_pids - prev_pids
        gone_pids = prev_pids - cur_pids

        # deal with gone pids
        if len(gone_pids) > 0:
            log.debug(f"find {len(gone_pids)} processes gone")
            for pid in gone_pids:
                cls.pid_set.remove(pid)
                if pid in cls.pid_app_dict:
                    log.debug(f"app process gone: {cls.pid_app_dict[pid]} (pid {pid})")
                    cls.pid_app_dict.pop(pid)

        # deal with new pids
        if len(new_pids) > 0:
            log.debug(f"find {len(new_pids)} processes created")
            for pid in new_pids:
                try:
                    process = psutil.Process(pid)
                    process_name = process.name()
                except psutil.NoSuchProcess:
                    continue
                cls.pid_set.add(pid)
                # match process name with known applications
                for app in cls.app_program_dict:
                    if process_name in cls.app_program_dict[app]:
                        cls.pid_app_dict[pid] = app
                        log.debug(f"app process created: {cls.pid_app_dict[pid]} (pid {pid})")
                        break

        # output current running applications
        cls._load["application"] = set(cls.pid_app_dict.values())
